"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonSize = exports.objectSize = exports.arraySize = exports.booleanSize = exports.stringSize = exports.numberSize = void 0;
const utf8_1 = require("../util/utf8");
const numberSize = (num) => JSON.stringify(num).length;
exports.numberSize = numberSize;
const stringSize = (str) => {
    const strLength = str.length;
    let byteLength = 0;
    let pos = 0;
    while (pos < strLength) {
        let value = str.charCodeAt(pos++);
        if (value < 128) {
            switch (value) {
                case 8:
                case 9:
                case 10:
                case 12:
                case 13:
                case 34:
                case 92:
                    byteLength += 2;
                    break;
                default:
                    byteLength += 1;
            }
            continue;
        }
        else
            return utf8_1.utf8Count(JSON.stringify(str));
    }
    return byteLength + 2;
};
exports.stringSize = stringSize;
const booleanSize = (bool) => (bool ? 4 : 5);
exports.booleanSize = booleanSize;
const arraySize = (arr) => {
    let size = 0;
    const length = arr.length;
    for (let i = 0; i < length; i++)
        size += exports.jsonSize(arr[i]);
    return size + 2 + (length > 1 ? length - 1 : 0);
};
exports.arraySize = arraySize;
const objectSize = (obj) => {
    let size = 2;
    let length = 0;
    for (const key in obj)
        if (obj.hasOwnProperty(key)) {
            length++;
            size += exports.stringSize(key) + exports.jsonSize(obj[key]);
        }
    const colonSize = length;
    const commaSize = length > 1 ? length - 1 : 0;
    return size + colonSize + commaSize;
};
exports.objectSize = objectSize;
const jsonSize = (value) => {
    if (value === null)
        return 4;
    switch (typeof value) {
        case 'number':
            return exports.numberSize(value);
        case 'string':
            return exports.stringSize(value);
        case 'boolean':
            return exports.booleanSize(value);
    }
    if (value instanceof Array)
        return exports.arraySize(value);
    return exports.objectSize(value);
};
exports.jsonSize = jsonSize;
