"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonSize = exports.objectSize = exports.arraySize = exports.booleanSize = exports.stringSize = exports.numberSize = void 0;
var utf8_1 = require("../util/utf8");
var numberSize = function (num) { return JSON.stringify(num).length; };
exports.numberSize = numberSize;
var stringSize = function (str) {
    var strLength = str.length;
    var byteLength = 0;
    var pos = 0;
    while (pos < strLength) {
        var value = str.charCodeAt(pos++);
        if (value < 128) {
            switch (value) {
                case 8:
                case 9:
                case 10:
                case 12:
                case 13:
                case 34:
                case 92:
                    byteLength += 2;
                    break;
                default:
                    byteLength += 1;
            }
            continue;
        }
        else
            return utf8_1.utf8Count(JSON.stringify(str));
    }
    return byteLength + 2;
};
exports.stringSize = stringSize;
var booleanSize = function (bool) { return (bool ? 4 : 5); };
exports.booleanSize = booleanSize;
var arraySize = function (arr) {
    var size = 0;
    var length = arr.length;
    for (var i = 0; i < length; i++)
        size += exports.jsonSize(arr[i]);
    return size + 2 + (length > 1 ? length - 1 : 0);
};
exports.arraySize = arraySize;
var objectSize = function (obj) {
    var size = 2;
    var length = 0;
    for (var key in obj)
        if (obj.hasOwnProperty(key)) {
            length++;
            size += exports.stringSize(key) + exports.jsonSize(obj[key]);
        }
    var colonSize = length;
    var commaSize = length > 1 ? length - 1 : 0;
    return size + colonSize + commaSize;
};
exports.objectSize = objectSize;
var jsonSize = function (value) {
    if (value === null)
        return 4;
    switch (typeof value) {
        case 'number':
            return exports.numberSize(value);
        case 'string':
            return exports.stringSize(value);
        case 'boolean':
            return exports.booleanSize(value);
    }
    if (value instanceof Array)
        return exports.arraySize(value);
    return exports.objectSize(value);
};
exports.jsonSize = jsonSize;
